@@ -1,5 +1,5 @@
 (*  Orpie -- a fullscreen RPN calculator for the console
- *  Copyright (C) 2003-2004, 2005, 2006-2007 Paul Pelzl
+ *  Copyright (C) 2003-2004, 2005, 2006-2007, 2010 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, Version 2,
@@ -15,7 +15,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *  Please send bug reports, patches, etc. to Paul Pelzl at 
- *  <pelzlpj@eecs.umich.edu>.
+ *  <pelzlpj@gmail.com>.
  *)
 
 (* interface_main.ml
@@ -533,9 +533,9 @@ let handle_exit_int (iface : interface_state_t) =
    iface.entry_type <- FloatEntry;
    iface.is_entering_base <- false;
    iface.int_base_string <- "";
-   if (String.length iface.gen_buffer.(0).re_mantissa > 0) or
-      (String.length iface.gen_buffer.(0).re_exponent > 0) or
-      (String.length iface.gen_buffer.(0).im_mantissa > 0) or
+   if (String.length iface.gen_buffer.(0).re_mantissa > 0) ||
+      (String.length iface.gen_buffer.(0).re_exponent > 0) ||
+      (String.length iface.gen_buffer.(0).im_mantissa > 0) ||
       (String.length iface.gen_buffer.(0).im_exponent > 0) then
       iface.has_entry <- true
    else
