@@ -1,5 +1,5 @@
 (*  Orpie -- a fullscreen RPN calculator for the console
- *  Copyright (C) 2003-2004, 2005, 2006-2007 Paul Pelzl
+ *  Copyright (C) 2003-2004, 2005, 2006-2007, 2010 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, Version 2,
@@ -15,7 +15,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *  Please send bug reports, patches, etc. to Paul Pelzl at 
- *  <pelzlpj@eecs.umich.edu>.
+ *  <pelzlpj@gmail.com>.
  *)
 
 (* rpc_calc.ml
@@ -390,9 +390,13 @@ class rpc_calc conserve_memory =
             if uu <> Units.empty_unit then begin
                stack#push gen_el;
                raise_invalid "cannot compute logarithm of dimensioned value"
-            end else
+            end else if el >= 0.0 then begin
                let (f, u) = funit_of_float (log el) in
                stack#push (RpcFloatUnit (f, u))
+            end else
+               let c_arg = c_of_f el in
+               let (c, u) = cunit_of_cpx (Gsl_complex.log c_arg) in
+               stack#push (RpcComplexUnit (c, u))
          |RpcComplexUnit (el, uu) ->
             if uu <> Units.empty_unit then begin
                stack#push gen_el;
@@ -458,9 +462,13 @@ class rpc_calc conserve_memory =
             if uu <> Units.empty_unit then begin
                stack#push gen_el;
                raise_invalid "cannot compute logarithm of dimensioned value"
-            end else
+            end else if el >= 0.0 then begin
                let (f, u) = funit_of_float (log10 el) in
                stack#push (RpcFloatUnit (f, u))
+            end else
+               let c_arg = c_of_f el in
+               let (c, u) = cunit_of_cpx (Gsl_complex.log10 c_arg) in
+               stack#push (RpcComplexUnit (c, u))
          |RpcComplexUnit (el, uu) ->
             if uu <> Units.empty_unit then begin
                stack#push gen_el;
@@ -1317,16 +1325,26 @@ class rpc_calc conserve_memory =
          |RpcInt el1 ->
             begin match gen_el2 with
             |RpcInt el2 ->
-               stack#push (RpcInt (mod_big_int el1 el2))
+                if eq_big_int el2 zero_big_int then begin
+                   stack#push gen_el1;
+                   stack#push gen_el2;
+                   raise (Invalid_argument "division by zero")
+                end else
+                   stack#push (RpcInt (mod_big_int el1 el2))
             |RpcFloatUnit (ff2, uu2) ->
                if uu2 <> Units.empty_unit then begin
                   stack#push gen_el1;
                   stack#push gen_el2;
                   raise (Invalid_argument "cannot compute mod of dimensioned values")
-               end else if (abs_float ff2) < 1e9 then begin
+               end else if (abs_float ff2) < 1e9 then
                   let bi_el2 = big_int_of_int (int_of_float ff2) in
-                  stack#push (RpcInt (mod_big_int el1 bi_el2))
-               end else begin
+                  if eq_big_int bi_el2 zero_big_int then begin
+                     stack#push gen_el1;
+                     stack#push gen_el2;
+                     raise (Invalid_argument "division by zero")
+                  end else
+                     stack#push (RpcInt (mod_big_int el1 bi_el2))
+               else begin
                   stack#push gen_el1;
                   stack#push gen_el2;
                   raise (Invalid_argument "real argument is too large to convert to integer")
@@ -1352,16 +1370,26 @@ class rpc_calc conserve_memory =
                let bi_el1 = big_int_of_int (int_of_float ff1) in
                begin match gen_el2 with
                |RpcInt el2 ->
-                  stack#push (RpcInt (mod_big_int bi_el1 el2))
+                  if eq_big_int el2 zero_big_int then begin
+                     stack#push gen_el1;
+                     stack#push gen_el2;
+                     raise (Invalid_argument "division by zero")
+                  end else
+                     stack#push (RpcInt (mod_big_int bi_el1 el2))
                |RpcFloatUnit (ff2, uu2) ->
                   if uu2 <> Units.empty_unit then begin
                      stack#push gen_el1;
                      stack#push gen_el2;
                      raise (Invalid_argument "cannot compute mod of dimensioned values")
-                  end else if (abs_float ff2) < 1e9 then begin
+                  end else if (abs_float ff2) < 1e9 then
                      let bi_el2 = big_int_of_int (int_of_float ff2) in
-                     stack#push (RpcInt (mod_big_int bi_el1 bi_el2))
-                  end else begin
+                     if eq_big_int bi_el2 zero_big_int then begin
+                        stack#push gen_el1;
+                        stack#push gen_el2;
+                        raise (Invalid_argument "division by zero")
+                     end else
+                        stack#push (RpcInt (mod_big_int bi_el1 bi_el2))
+                  else begin
                      stack#push gen_el1;
                      stack#push gen_el2;
                      raise (Invalid_argument "real argument is too large to convert to integer")
